      PROGRAM MAIN
C------------------------------------------------------------------------
C  POMGL Version 2007 (based on pom2K)
C   D.J. Schwab and D. Beletsky
C   NOAA Great Lakes Environmental Research Laboratory
C   Ann Arbor, Michigan
C------------------------------------------------------------------------
C  Version 2.1 - 3/97  (note that IPSWR switched to nbct in 11/2004).
C   - Included provision for time-dependent TMEAN.  Input parameter
C      ITMEAN determines how often TMEAN is recalculated.  Default
C      is ITMEAN=0 (no recalculation). If ITMEAN <> 0, then the mean
C      temperature profiles calculated by subroutine TZAVG are written
C      to XDR file FOUT.tmz.
C   - Included input parameter nbct(formerly IPSWR) to indicate surface
C      boundary condition for subroutine PROFT.  Default is nbct=1.
C      nbct=1: SURF. B.C. IS WFSURF. NO SW RADIATIVE PENETRATION.
C      nbct=2; SURF. B.C. IS WFSURF+SWRAD*(1.-TR). TR*SWRAD PENETRATES
C                WATER COLUMN
C      nbct=3; SURF. B.C. IS TSURF. NO SW RADIATIVE PENETRATION.
C      nbct=4; SURF. B.C. IS TSURF. TR*SWRAD PENETRATES WATER COLUMN
C   - Temperature is limited to a minimum of 0 C.
C   - Added provision for creating output fields of 2-d and 3-d
C      currents at grid square centers.
C   - Included input parameter for surface fluxes specification for idealized
C      cases (ISFLUX). If ISFLUX=0, then wind stress and surface heat flux
C      are calculated from meteorological data. If ISFLUX=1, then surface
C      fluxes are specified directly. If ISFLUX=2, then wind stress is 
C      calculated, but heat flux is specified. If ISFLUX=3, then TAU=CONST,
C      HF=0. Default is TAU=0. For steady-state calculations add TAUX and TAUY 
C      to the input file. If ISFLUX=4, then wind stress is calculated.
C
C      ISFLUX  SUBR   WIND     MODE   INPUT FILES     GENERATED BY
C
C        0     FLUX1  REAL     2,4    wu,wv,at,wt      interp.f;pomglflx2.f 
C                              3      wu,wv,at,dp,cl
C
C        1     FLUX2  IDEAL    2,4    tx,ty            pomglflx.f 
C                              3      tx,ty,hf,swr
C
C        2     FLUX3  REAL     2,4    wu,wv,at,wt      interp.f;pomglflx2.f 
C                              3      wu,wv,at,hf,swr
C
C        3     FLUX4  CONST    2,3,4  None             Wind stress=1dyn/cm2
C                                                          TAUX       TAUY
C                                     See examples ->  N    0.        1.E-4      
C                                                      NNE  0.383E-4  0.924E-4	
C                                                      NE   0.707E-4  0.707E-4
C                                                      NEE  0.924E-4  0.383E-4
C                                                      E    1.E-4     0.
C                                                      ESE  0.924E-4 -0.383E-4
C                                                      SE   0.707E-4 -0.707E-4
C                                                      SSE  0.383E-4 -0.924E-4	
C
C        4     FLUX5  REAL     3    wu,wv,at           interp.f;pomglflx2.f 
C                                   (zero heat flux)          
C
C   - Included input parameter for constant vertical/horizontal viscosity 
C      and diffusivity coefficients. Default is SMAGOR=0.,YAMADA=0.
C      If SMAGOR > 0., then AAM=SMAGOR, horiz.diffusivity is always AAM/TPRNU
C      If YAMADA > 0., then KM=YAMADA, KH=YAMADA1 (constant KM and KH) 
C   - Output instantaneous fields UA,VA, and EL replaced by the time-average
C     of UTB-UTF,VTB-VTF, and EGB-EGF to match internal mode output more 
C     precisely. 
C   - In the control file IDAYS is replaced by IHRS.
C
C  Version 97.1 - 5/97
C   - Added provision for multiple internal time step average fields
C      in output (UTM, VTM, UM, VM, UCM, VCM)
C   - Added input parameter IAVG = number of internal time steps for
C      time averaging of output fields (UTM, VTM, UM, VM, UCM, VCM)
C   - Improved management of output specifications in subroutines
C      READINPUT and OUTP3D
C
C  Version 98.1 - 6/98; 4/99
C   - Old PROFQ is replaced by a new one (PROFQ_97)
C   - Corrected calculation of IEND (number of internal time steps in the
C      model run). Call to OUTP3D is now made AFTER updating time to correct
C      time in the standard output file .out 
C
C  Version 99.1 - 5/99
C   - Corrected functions xdist,ydist (portions that deal with the Lambert
C      conformal projection).
C   - Several generic subroutines were moved from the file pomgl-subs.f into
C      separate directory. These subroutines are used by several GLERL models. 
C
C  Version 2000 - 3/00; 6/00; 10/00; 3/01; 4/01; 1/02; 7/2003
C   - Extended treatment of idealized cases with ISFLUX=3 to specify zero wind
C      stress and heat flux without storing them in XDR files. 
C   - Subroutine bcond.f was removed from the file pom-subs.f and is now
C      compiled as a stand alone subroutine (in file bcond.f) in no open
C      boundaries cases. In cases with river discharges it is contained in
C      files bcond_riversEEGLE.f (Lake Michigan, 2 km grid) and 
C      bcond_riversLMMBS.f (Lake Michigan, 5 km grid).
C   - Fixed a bug in bcond_riversEEGLE.f and bcond_riversLMMBS.f pointed out
C      by Ray Chapman.
C   - Added provision for time-dependent but vertically uniform KM and KH.
C      KM is dependent on wind stress magnitude only (see Simons, JPO,1974)
C      To use this parameterization specify NEGATIVE YAMADA parameter:
C      KM=0.0025+ABS(YAMADA)*|WUSURF|*DTE.    KH=KM/10.
C      Example: For |WUSURF|=0.0001 , DTE=10  and YAMADA=-10  KM=0.0125 m2/s. 
C      Note: When YAMADA is positive, it is equal to KM, when it is negative
C            it is just a coefficient in the formula above.
C   - Extended treatment of idealized cases with ISFLUX=3 to specify constant
C      wind stress and zero heat flux without storing them in XDR files. FINIT
C      field may be left blank.  
C   - Extended treatment of realistic cases with ISFLUX=4 to specify wind
C      and air temperature. Heat flux is zero. Added parameter INEUTR (FLUX5 only).
C      Default is INEUTR=0. If INEUTR is not zero, wind stress is calculated
C      for neutral stratification.
C   - Added provision for saving multiple initial conditions in the multiple
C      restart file. Parameter NRSTRT (in internal time steps) determines how
C      often fields are saved in a file. Default is NRSTRT, file .mrst is not
C      created then. 
C   - Added provision to skip a number of records (NRSKIP) in the met.data files.
C      Default is NRSKIP=0. To specify NRSKIP, always check DTWIND first. 
C   - Added 2dx noise diagnostics in the output file.
C   - Added provision (hard-wired) to smooth KH horizontally to reduce 2dx noise
C      in temperature field. This was later removed (11/2004).
C   - Restart file now includes fluxes for a seamless restart.
C
C  Version 2004 - 11/2004
C   - Major revision with a goal to upgrade POMGL to pom2k version.
C      Added several input parameters for selection of advection schemes, 
C      salinity surface fluxes and optical properties. IPSWR parameter is now 
C      called "nbct"  and IRSTRT parameter is now called "nread" to comply with
C      pom2k notation.
C
C  Version 2007 - 3/2007
C   - Most changes are due to nested grid option to improve model stability:
C     depth-dependent Zo, limitations on vertical diffusion coefficients are
C     added. New NTP=6 added for Lake Erie case. Horizontal viscosity
C     in 2D cases (aam_init) reduced to 20. Added provision to savw Q2,L,KMM,KHM
C   - Changes made to include transverse velocity points in dum and dvm for
C     j=1 and i=1 for nested grids.  Also changed loop limits on utf and vtf.(DJS)
C-----------------------------------------------------------------------
C
C Purpose :
C  The purpose of this program is to implement the Princeton numerical
C   Ocean Model for the Great Lakes and idealized basins.
C
C  The Fortran code for this model consists of four major files and several
C   subroutines (stored in the directory glerl-lib) which are used by other
C   GLERL models (WMGL, PARTICGL,STMGL,POMGL1D) and other programs:
C
C   1) pomgl20XX-main.f: The main program.  This is basically the main program
C       from the Princeton Ocean Model customized for the Great Lakes.
C   2) pomgl20XX-subs.f: GLERL subroutines for input, output, and specification
C       of surface fluxes.
C   3) pom20XX-subs.f: Subroutines from the Princeton Ocean Model.
C   4) pomgl20XX.h: Include file for common blocks.  This is a combination
C       of comblk.h from the Princeton Ocean Model with additional
C       variables for GLERL additions.
C   5) rgrid.f,rlat.f,rlon.f, xdist.f,ydist.f: GLERL bathymetric grid and 
C       coordinate conversion subroutines;
C      wxdr2.f.rxdr2d.f: GLERL subroutines handling conversion between 2-d
C       and 1-d arrays needed in I/O of XDR files;
C      uz.f,uzl.f,uzl10.f: GLERL subroutines handling calculation of surface
C       fluxes.      
C   6) bcond20XX.f: boundary conditions subroutine
C
C
C  In this application of the Princeton Ocean Model, input and
C  output functions have been customized for use with Great Lakes
C  bathymetric grids.  The format of the Great Lakes grids is
C  described in:
C
C      Schwab, D.J. and D.L. Sellers, Computerized Bathymetry and Shorelines
C      of the Great Lakes, NOAA Data Report ERL-GLERL-16, 1980.
C  and
C      Schwab, D.J., J.R. Bennett, and A.T. Jessup, A Two-dimensional
C      Lake Circulation Modeling System, NOAA Technical Memorandum
C      ERL-GLERL-38, 1981.
C   
C  Input for controlling model runs, specifying model parameters,
C  and specifying required output data is assumed to be present on
C  STDIN (Logical unit 5 or redirected input file).  The format of this file is:
C
C  Record 1: [POMGL CONTROL] = Identifier for control file
C  Record 2: FGRID  = Full pathway name for bathymetry file  
C  Record 3: FINPUT = Full pathway name for prefix of meteorological data files 
C  Record 4: FOUT   = Full pathway name for prefix of output data files
C  Record 5: FINIT  = Full pathway name for initialization (or restart) file
C                       (or blank for no special initialization, in this case
C                         To=10C, S=35%o, u=v=w=0)
C  Record 6: MODE   = 2; 2-D calculation (bottom stress calculated in 
C                        subroutine ADVAVE)  
C                     3; 3-D claculation (bottom stress calculated in 
C                        subroutines PROFU, PROFV)  
C                     4; 3-D calculation with T and S held fixed
C     For MODE=2,4    Files FINPUT.wu, FINPUT.wv, FINPUT.at, FINPUT.wt   
C                      read by subroutines FLUX1-3 and used to calculate  
C                      surface wind stress. Files FINPUT.dp and  
C                      FINPUT.cl are not used. No files are used by FLUX4.
C     For MODE=3      Files FINPUT.wu, FINPUT.wv, FINPUT.at, FINPUT.dp, 
C                      and FINPUT.cl are read by subroutines FLUX1-3 and  
C                      used to calculate wind stress and heat flux.    
C                      The file FINPUT.wt is not used. No files are used by FLUX4.                             
C  Record 7:  IYR,IJD,IHR     
C                    = Year, day, and hour at start of run (e.g. 1967,220,13)
C  Record 8:  DTE    = External mode time step in seconds
C  Record 9:  DTI    = Internal mode time step in seconds  
C  Record 10: IHRS   = Length of run in hours 
C  Record 11: DTWIND = Time interval in hours at which new meteorological
C                       fields are read from input data files
C  Record 12: NLEVEL = Integer to determin if user supplies Z(KB) 
C                       vertical level distribution or if program computes it
C                      NLEVEL=0; Z(KB) determined by subroutine DEPTH
C                      NLEVEL>0; User supplies Z(KB) values   
C  Record 13: Z(KB)  = User supplied vertical levels (If NLEVEL>0) 
C  Record 14: [POMGL CONSTANTS] = Identifier for end of control parameters
C                    and beginning of model parameters.  If a model parameter
C                    is not specified, its default value is used in the model.
C  Record 15 (and following): Model parameters.  The following model parameters
C                    can be specified (default values are indicted here):
C                     ISPADV=1
C                     HORCON=0.1
C                     tprni=1.0
C                     SMOTH=0.1
C                     UMOL=2.E-5
C                     TBIAS=10.0
C                     SBIAS=35.0
C                     ALPHA=0.225
C                     RAMPDT=0.
C                     aam_init=20.0
C                     DADD=0.0
C                     CBCMIN=0.0025
C                     nread=0
C                     nbct=1
C                     ITMEAN=0
C                     ISFLUX=0
C                     TAUX=0
C                     TAUY=0
C                     SMAGOR=0.
C                     YAMADA=0.
C                     IAVG=1
C                     INEUTR=0 (FLUX5 only)
C                     NRSTRT=0
C                     NRSKIP=0
C                     nadv=1
C                     nitera=2
C                     sw=0.5e0
C                     ntp=2
C                     nbcs=1
C                     ICE=0
C
C                   To override the default value, specify the name of the
C                   parameter (in caps) followed by an equal sign (no spaces)
C                   followed by the new parameter value.
C  Record 16 (or next record after model parameter records): [POMGL OUTPUT]
C                   = identifier for end of model parameters section and
C                   beginning of output specifiers section.
C  Record 17 (or next record) = Output specifier.  An output specifier
C                   describes what field is required, whether the entire
C                   field, or an i, j, or k slice (for 3-d fields), or
C                   a time series at a point (for 2-d and 3-d fields)
C                   is required, and the time interval (in internal
C                   mode time steps) at which it is required.
C                   The format for the output specifier record is:
C                    fff iii jjj kkk ttttt
C                   Where
C                    fff = Field name, one of the following:
C                         2-d fields:
C                           UT = vertically averaged u velocity (at u point)
C                           VT = vertically averaged v velocity (at v point)
C                           UTM= time averaged, vertically averaged u
C                             velocity (at u point)
C                           VTM= time averaged, vertically averaged v
C                             velocity (at v point)
C                           EG = water surface elevation
C                           HF = net surface heat flux
C                           SHF= sensible heat flux
C                           LHF= latent heat flux
C                           SWR= short wave radiation
C                           LWR= long wave radiation
C                           TX = x component of wind stress
C                           TY = y component of wind stress
C                           UTC= vertically averaged u velocity (at grid center)
C                           VTC= vertically averaged v velocity (at grid center)
C                         3-d fields:
C                           TM = Mean (background) temperature field
C                           SM = Mean (background) salinity field
C                           U  = u velocity (at u point)
C                           V  = v velocity (at v point)
C                           UM = time averaged u velocity (at u point)
C                           VM = time averaged v velocity (at v point)
C                           W  = vertical velocity
C                           T  = temperature
C                           S  = salinity
C                           KM = vertical diffusivity for momentum
C                           KH = vertical diffusivity for heat
C                           KMM = time everaged vertical diffusivity for momentum
C                           KHM = time everaged vertical diffusivity for heat
C                           AAM= horizontal diffusivity
C                           UC = u velocity (at grid center)
C                           VC = v velocity (at grid center)
C                           UCM= time averaged u velocity (at grid center)
C                           VCM= time averaged v velocity (at grid center)
C                    iii = i-index of grid cell
C                    jjj = j-index of grid cell
C                    kkk = k-index of grid cell
C                         If iii=jjj=kkk=0, then full 2-d or 3-d field is saved.
C                         If iii, jjj, and kkk are all nonzero, then
C                          a time series at that grid point is saved.
C                         If iii and jjj are nonzero and kkk is zero,
C                          a time series of all kb levels at that
C                          gridpoint is saved.
C                         For 3-d fields only:
C                          if iii is nonzero and jjj=kkk=0, then
C                           an i-slice (dimensions jm,kb) is saved
C                          if jjj is nonzero and iii=kkk=0, then
C                           a j-slice (dimensions im,kb) is saved
C                          if kkk is nonzero and iii=jjj=0, then
C                           a k-slice (dimensions im,jm) is saved,
C                           if k greater than 100, then data saved using mask
C                           arrays          
C                     ttttt = time increment (number of internal
C                      time steps between all output fields except
C                      UTM,VTM,UM,VM,UCM,VCM)
C
C                     The output specifier record also determines the
C                     name of the output file as follows:
C                      If iii=jjj=kkk=0, then the output file has the
C                       file name specified by FOUT with the file extension
C                       given by the field name (from above) in lower
C                       case letters, i.e., FOUT.ua
C                      If iii or jjj or kkk is nonzero, then the filename
C                       is FOUT.fff.iii.jjj.kkk where fff is the field
C                       name in lower case letters.
C
C NOTE:  All output files are in XDR format.  XDR format files for
C        full 2-d and 3-d fields contain data only for those grid points
C        actually in the lake domain, with non-zero elements of 
C        mask arrays FSM(I,J), DUM(I,J), or DVM(I,J).  Date is
C        stored in order of increasing I, increasing j, and
C        increasing k (for 3-d fields).  Conversion between 2-d
C        and 1-d arrays is done in subroutines RXDR2D AND WXDR2. 
C **********************************************************************
C *                                                                    *
C *   The last code change as rcorded in pom2k.change was on           *
C *                                                                    *
C *                     2004-01-08                                     *
C *                                  (adding IC from file)             *
C *                                                                    *
C * FUNCTION    :  This is a version of the three dimensional, time    *
C *                dependent, primitive equation, ocean model          *
C *                developed by Alan Blumberg and George Mellor with   *
C *                subsequent contributions by Leo Oey, Steve Brenner  *
C *                and others. It is now called the Princeton Ocean    *
C *                Model. Two references are:                          *
C *                                                                    *
C *                Blumberg, A.F. and G.L. Mellor; Diagnostic and      *
C *                  prognostic numerical circulation studies of the   *
C *                  South Atlantic Bight, J. Geophys. Res. 88,        *
C *                  4579-4592, 1983.                                  *
C *                                                                    *
C *                Blumberg, A.F. and G.L. Mellor; A description of a  *
C *                  three-dimensional coastal ocean circulation model,*
C *                  Three-Dimensional Coastal Ocean Models, Coastal   *
C *                  and Estuarine Sciences, 4, N.S. Heaps, ed.,       *
C *                  American Geophysical Union, 1-16, 1987.           *
C *                                                                    *
C *                In subroutine profq the model makes use of the      *
C *                turbulence closure sub-model described in:          *
C *                                                                    *
C *                Mellor, G.L. and T. Yamada; Development of a        *
C *                  turbulence closure model for geophysical fluid    *
C *                  problems, Rev. Geophys. Space Phys., 20, No. 4,   *
C *                  851-875, 1982.                                    *
C *            (note recent profq that includes breaking waves)        *
C *                                                                    *
C *                A user's guide is available:                        *
C *                                                                    *
C *                Mellor, G.L.; User's guide for a three-dimensional, *
C *                  primitive equation, numerical ocean model.        *
C *                  Princeton University Report, 1998.                *
C *                                                                    *
C *                In October 2001, the source code underwent          *
C *                revision by John Hunter of the University of        *
C *                Tasmania. Major aspects of the revision were:       *
C *                                                                    *
C *                (1) The revision was based on pom98 updated to      *
C *                    12/9/2001.                                      *
C *                (2) Declaration of all variables.                   *
C *                (3) Rationalisation of the input of all constants.  *
C *                (4) Modifications to the "printer" output.          *
C *                (5) Output to a netCDF file.                        *
C *                (6) Inclusion of surface freshwater flux.           *
C *                (7) Inclusion of atmospheric pressure.              *
C *                (8) Inclusion of an additional problem to check (6) *
C *                    and (7), above.                                 *
C *                (9) Inclusion of option for Smolarkiewicz           *
C *                    advection scheme.                               *
C *                                                                    *
C *                This revised version is functionally almost         *
C *                equivalent to pom98. The output to device 6 from    *
C *                the "seamount" problem should be almost the same,   *
C *                any differences being due to minor format changes   *
C *                and improvements in rounding.                       *
C *                                                                    *
C *                This revision was helped by the following people:   *
C *                Tal Ezer, Peter Holloway, George Mellor, Rich       *
C *                Signell, Ian Webster, Brian Williams and Emma Young.*
C *                                                                    *
C **********************************************************************
C *                                                                    *
C *                                  GENERAL NOTES                     *
C *                                                                    *
C *                1. All units are S.I. (M.K.S.) unless otherwise     *
C *                   stated. NOTE that time is in days from the start *
C *                   of the run.                                      *
C *                                                                    *
C *                2. "b", <nothing> and "f" refers to backward,       *
C *                   central and forward time levels.                 *
C *                                                                    *
C *                3. NetCDF output may be used. In order to omit/use  *
C *                   netCDF, comment/uncomment all statements         *
C *                   carrying the comment "*netCDF*" at the end of    *
C *                   the line (or set netcdf_file='nonetcdf')         *
C *                                                                    *
C *                4. NetCDF is version 3. An attempt has been made to *
C *                   conform to the NetCDF Climate and Forecast (CF)  *
C *                   Metadata Conventions, but this may not yet be    *
C *                   complete (see:                                   *
C *                                                                    *
C *          http://www.cgd.ucar.edu/cms/eaton/cf-metadata/index.html) *
C *                                                                    *
C *                5. In order to use netCDF, the program should be    *
C *                   compiled with the appropriate library. For       *
C *                   example, if using g77, you may need to type:     *
C *                                                                    *
C *                     g77 -o pom2k pom2k.f /usr/lib/libnetcdf.a      *
C *                                                                    *
C *                   You should also have the "include" file of       *
C *                   netCDF subroutines (pom2k.n).                    * 
C *                                                                    *
C *                6. In order to use netCDF, you may need to change   *
C *                   the name of the "include" file in the statement: *
C *                                                                    *
C *                     include '/usr/include/netcdf.inc'              *
C *                                                                    *
C *                   in subroutine write_netcdf                       *
C *                                                                    *
C **********************************************************************
C *                                                                    *
C *                                SOFTWARE LICENSING                  *
C *                                                                    *
C *                This program is free software; you can redistribute *
C *                it and/or modify it under the terms of the GNU      *
C *                General Public License as published by the Free     *
C *                Software Foundation, either Version 2 of the        *
C *                license, or (at your option) any later version.     *
C *                                                                    *
C *                This program is distributed in the hope that it     *
C *                will be useful, but without any warranty; without   *
C *                even the implied warranty of merchantability or     *
C *                fitness for a particular purpose. See the GNU       *
C *                General Public License for more details.            *
C *                                                                    *
C *                A copy of the GNU General Public License is         *
C *                available at http://www.gnu.org/copyleft/gpl.html   *
C *                or by writing to the Free Software Foundation, Inc.,*
C *                59 Temple Place - Suite 330, Boston, MA 02111, USA. *
C *                                                                    *
C **********************************************************************

C-----------------------------------------------------------------------
C
C     Advection scheme:
C
C      nadv     Advection scheme
C
C        1       Centred scheme, as originally provide in POM
C        2       Smolarkiewicz iterative upstream scheme, based on
C                subroutines provided by Gianmaria Sannino and Vincenzo
C                Artale
C
C-----------------------------------------------------------------------
C
C     Constants for Smolarkiewicz iterative upstream scheme.
C
C     Number of iterations. This should be in the range 1 - 4. 1 is
C     standard upstream differencing; 3 adds 50% CPU time to POM:
C
C     Smoothing parameter. This should preferably be 1, but 0 < sw < 1
C     gives smoother solutions with less overshoot when nitera > 1:
C
C-----------------------------------------------------------------------
C
C     Water type, used in subroutine proft.
C
C       ntp    Jerlov water type
C
C        1            i
C        2            ia
C        3            ib
C        4            ii
C        5            iii
C        6            Lake Erie
C
C-----------------------------------------------------------------------
C
C     Surface temperature boundary condition, used in subroutine proft:
C
C       nbct   prescribed    prescribed   short wave
C              temperature      flux      penetration
C
C        1        no           yes           no
C        2        no           yes           yes
C        3        yes          no            no
C        4        yes          no            yes
C
C-----------------------------------------------------------------------
C
C     Surface salinity boundary condition, used in subroutine proft:
C
C       nbcs   prescribed    prescribed
C               salinity      flux
C
C        1        no           yes
C        3        yes          no
C
C     NOTE that only 1 and 3 are allowed for salinity.
C-----------------------------------------------------------------------

      implicit none
 
      INCLUDE 'pomgl2007.h'    

C
C     New declarations plus ispi,isp2i:
C
      real cbcmax,cflmax
      real days,dte2,dvol,dayi
      real ispi,isp2i
      real om
      real rlat
      real time0
      real vmax1
      real z0b,zb
      integer i,iend,iext,imax,iswtch,ip,isk,iskip,itime
      integer j,jmax
      integer k
      integer ndyr

C Set constants

      PI=3.1416E0
      RAMP=1.E0
      SMALL=1.E-10
      TIME0=0.E0
      GRAV=9.806E0
      UMOL=2.E-5
      OM=7.292E-5
      kappa=0.4
      rhoref=1000.e0
      cbcmax=1.e0
      vmax1=100.e0     

C-----------------------------------------------------------------------
C
C     Integers defining the number of logarithmic layers at the   
C     surface and bottom (used by subroutine depth). The number of
C     logarithmic layers are kl1-2 at the surface and kb-kl2-1
C     at the bottom. For no log portions, set kl1=2 and kl2=kb-1:
C
      kl1=6
      kl2=kb-2
C
C-----------------------------------------------------------------------

C CALL SUBROUTINE INITIALIZE TO SET ALL ARRAYS IN THE COMMON  
C BLOCK 'pomgl20XX.h' AND IN THE MAIN PROGRAM TO ZERO  

      CALL INITIALIZE 

      IINT=0
      TIME=0.

C  CALL SUBROUTINE TO READ PROBLEM PARAMETERS FROM INPUT FILE
C     UNIT = 5 

      CALL READINPUT

      hmax=depmax+dadd
      ISPLIT=DTI/DTE
      DTE2=DTE*2
      DTI2=DTI*2
      IEND=IHRS*3600/IFIX(DTI)

      DAYI=1.E0/86400.E0
      ISPI=1.E0/FLOAT(ISPLIT)
      ISP2I=1.E0/(2.E0*FLOAT(ISPLIT))

C DETERMINE MAXIMUM TIME STEP FOR CFL CRITERION 

      CFLMAX=0.5E0/SQRT(1.E0/DS**2+1.E0/DS**2)
     1         /SQRT(GRAV*DEPMAX)
      WRITE(6,7046) CFLMAX
 7046 FORMAT(/' MAXIMUM CFL TIME STEP = ',F10.2) 

C----------------------------------------------------------------------
C             ESTABLISH PROBLEM CHARACTERISTICS
C          ****** ALL UNITS IN M.K.S. SYSTEM ******
C      F,BLANK AND B REFERS TO FORWARD,CENTRAL AND BACKWARD TIME LEVELS.
C----------------------------------------------------------------------
C
      DO 12 J=1,JM
      DO 12 I=1,IM
      DX(I,J)=DS    
      DY(I,J)=DS
      ART(I,J)=DX(I,J)*DY(I,J)
  12  CONTINUE 
      DO 13 J=2,JM
      DO 13 I=2,IM
      ARU(I,J)=.25*(DX(I,J)+DX(I-1,J))*(DY(I,J)+DY(I-1,J))
  13  ARV(I,J)=.25*(DX(I,J)+DX(I,J-1))*(DY(I,J)+DY(I,J-1))
      DO 121 J=1,JM
      ARU(1,J)=ARU(2,J)
      ARV(1,J)=ARV(2,J)
  121 CONTINUE 
      DO 122 I=1,IM
      ARU(I,1)=ARU(I,2)
      ARV(I,1)=ARV(I,2)
  122 CONTINUE  

C CALCULATE CORIOLIS PARAMETER USING FUNCTION RLAT 

      DO 14 J=1,JM
      DO 14 I=1,IM
      COR(I,J)=2.0*OM*         
     1         SIN((RLAT((I-0.5)*DS,(J-0.5)*DS))*PI/180.0)  
  14  CONTINUE

C SET TMEAN AND SMEAN TO THE INITIAL T AND S FIELDS

      DO 17 K=1,KB
      DO 17 J=1,JM
      DO 17 I=1,IM
      TMEAN(I,J,K)=TB(I,J,K)
      SMEAN(I,J,K)=SB(I,J,K)
  17  CONTINUE

      DO 16 J=1,JM
      DO 16 I=1,IM
      TSURF(I,J)=TB(I,J,1)
  16  SSURF(I,J)=SB(I,J,1)

C SET COEFF OF BOTTOM FRICTION FOR GREAT LAKES  

      DO 45 J=1,JM
      DO 45 I=1,IM
      IF(MODE.EQ.2) THEN 
	 CBC(I,J)=CBCMIN
      ELSE
         Z0B=0.001+0.02/(1.+H(I,J))
	 ZB=AMAX1(2.*Z0B,(ZZ(KBM1)-Z(KB))*H(I,J)) 
	 CBC(I,J)=AMAX1(CBCMIN,0.16/ALOG(ZB/Z0B)**2)  
      ENDIF   
   45 CONTINUE   

      DO 46 J=1,JM
      DO 46 I=1,IM
      CBC(I,J)=CBC(I,J)*FSM(I,J)
   46 CONTINUE    

      IP=IM
      ISK=1

C Initial conditions:

      do i=1,im
        do j=1,jm
          ua(i,j)=uab(i,j)
          va(i,j)=vab(i,j)
          el(i,j)=elb(i,j)
          et(i,j)=etb(i,j)
          etf(i,j)=et(i,j)
          d(i,j)=h(i,j)+el(i,j)
          dt(i,j)=h(i,j)+et(i,j)
          w(i,j,1)=vfluxf(i,j)
        end do
      end do
C
      do k=1,kb
        do j=1,jm
          do i=1,im
            l(i,j,k)=0.1*dt(i,j)
            q2b(i,j,k)=small
            q2lb(i,j,k)=l(i,j,k)*q2b(i,j,k)
            kh(i,j,k)=l(i,j,k)*sqrt(q2b(i,j,k))
            km(i,j,k)=kh(i,j,k)
            kq(i,j,k)=kh(i,j,k)
            aam(i,j,k)=aam_init
          end do
        end do
      end do
C
      do k=1,kbm1
        do i=1,im
          do j=1,jm
            q2(i,j,k)=q2b(i,j,k)
            q2l(i,j,k)=q2lb(i,j,k)
            t(i,j,k)=tb(i,j,k)
            s(i,j,k)=sb(i,j,k)
            u(i,j,k)=ub(i,j,k)
            v(i,j,k)=vb(i,j,k)
          end do
        end do
      end do
C


      call dens(s,t,rho)

C  IF FINIT IS PRESENT, THEN INITIALIZE MODEL FROM RESTART FILE 

      IF(FINIT.NE.' ') CALL RSTART(0)

      DO 80 K=1,KBM1
      DO 80 J=1,JM
      DO 80 I=1,IM
  80  RMEAN(I,J,K)=RHO(I,J,K)   

      call baropg

C
      do k=1,kbm1
        do j=1,jm
          do i=1,im
            drx2d(i,j)=drx2d(i,j)+drhox(i,j,k)*dz(k)
            dry2d(i,j)=dry2d(i,j)+drhoy(i,j,k)*dz(k)
          end do
        end do
      end do
C


      IF(YAMADA.GT.0.) THEN
        DO 53 K=1,KB
        DO 53 J=1,JM
        DO 53 I=1,IM
        KM(I,J,K)=YAMADA
  53    KH(I,J,K)=YAMADA1
      ENDIF

      DO 81 J=1,JM
      DO 81 I=1,IM
      D(I,J)=H(I,J)+EL(I,J)
      DT(I,J)=H(I,J)+ET(I,J)
   81 aam2d(i,j)=aam(i,j,1)

C  USE IYR0, IJD0, AND IHR0 AS COUNTERS

      IYR0=IYR
      IJD0=IJD
      IHR0=IHR
      ISEC=0
      IF(MOD(IYR0,4).EQ.0) THEN
       NDYR=366
      ELSE
       NDYR=365
      ENDIF

C***********************************************************************
C                                                                      *
C                  BEGIN NUMERICAL INTEGRATION                         *
C                                                                      *
C***********************************************************************

                     DO 9000 IINT=1,IEND

C  ITIME = TIME IN SECONDS FROM BEGINNING OF RUN
C  TIME = TIME IN HOURS FROM BEGINNING OF RUN

      ITIME=DTI*IINT
      TIME=ITIME/3600.

      RAMP=1.
      IF(RAMPDT.NE.0.AND.FLOAT(ITIME).LE.RAMPDT)
     1    RAMP=FLOAT(ITIME)/RAMPDT

C------------------------------------------------------------------------
C     SET TIME DEPENDENT, SURFACE AND LATERAL BOUNDARY CONDITIONS.
C     THE LATTER WILL BE USED IN SUBROUTINE BCOND .
C------------------------------------------------------------------------

C GET WIND STRESS AND HEAT FLUX FIELDS
C  WIND STRESS VALUE IS NEGATIVE FOR WESTERLY AND SOUTHERLY WINDS

cccgl      IF(ISFLUX.EQ.0) CALL FLUX1
      IF(ISFLUX.EQ.0.AND.ICE.EQ.0) CALL FLUX1
      IF(ISFLUX.EQ.0.AND.ICE.EQ.1) CALL FLUX6
      IF(ISFLUX.EQ.1) CALL FLUX2
      IF(ISFLUX.EQ.2) CALL FLUX3
      IF(ISFLUX.EQ.3) CALL FLUX4
      IF(ISFLUX.EQ.4) CALL FLUX5

C-----------------------------------------------------------------------
C
C     Set lateral viscosity:
C
C     If mode=2 then initial values of aam2d are used. If one wishes
C     to use Smagorinsky lateral viscosity and diffusion for an
C     external (2-D) mode calculation, then appropiate code can be
C     adapted from that below and installed just before the end of the
C     "if(mode.eq.2)" loop in subroutine advave.
C
C     Calculate Smagorinsky lateral viscosity:
C
C       ( hor visc = horcon*dx*dy*sqrt((du/dx)**2+(dv/dy)**2
C                                     +.5*(du/dy+dv/dx)**2) )
C
        if(mode.ne.2) then
          call advct(a,c,ee)
          call baropg
C
          do k=1,kbm1
            do j=2,jmm1
              do i=2,imm1
        IF(SMAGOR.EQ.0.) THEN
                aam(i,j,k)=horcon*dx(i,j)*dy(i,j)
     $                      *sqrt( ((u(i+1,j,k)-u(i,j,k))/dx(i,j))**2
     $                            +((v(i,j+1,k)-v(i,j,k))/dy(i,j))**2
     $                      +.5e0*(.25e0*(u(i,j+1,k)+u(i+1,j+1,k)
     $                                   -u(i,j-1,k)-u(i+1,j-1,k))
     $                      /dy(i,j)
     $                      +.25e0*(v(i+1,j,k)+v(i+1,j+1,k)
     $                             -v(i-1,j,k)-v(i-1,j+1,k))
     $                      /dx(i,j)) **2)
        ELSE
          AAM(I,J,K)=SMAGOR
        ENDIF 

              end do
            end do
          end do
C
C     Form vertical averages of 3-D fields for use in external (2-D)
C     mode:
C
          do j=1,jm
            do i=1,im
              adx2d(i,j)=0.e0
              ady2d(i,j)=0.e0
              drx2d(i,j)=0.e0
              dry2d(i,j)=0.e0
              aam2d(i,j)=0.e0
            end do
          end do
C
          do k=1,kbm1
            do j=1,jm
              do i=1,im
                adx2d(i,j)=adx2d(i,j)+advx(i,j,k)*dz(k)
                ady2d(i,j)=ady2d(i,j)+advy(i,j,k)*dz(k)
                drx2d(i,j)=drx2d(i,j)+drhox(i,j,k)*dz(k)
                dry2d(i,j)=dry2d(i,j)+drhoy(i,j,k)*dz(k)
                aam2d(i,j)=aam2d(i,j)+aam(i,j,k)*dz(k)
              end do
            end do
          end do
C
          call advave(tps)
C
          do j=1,jm
            do i=1,im
              adx2d(i,j)=adx2d(i,j)-advua(i,j)
              ady2d(i,j)=ady2d(i,j)-advva(i,j)
            end do
          end do
C
        endif
C
        do j=1,jm
          do i=1,im
            egf(i,j)=el(i,j)*ispi
          end do
        end do
C
        do j=1,jm
          do i=1,im
           if(i.gt.1)
     $       utf(i,j)=ua(i,j)*(d(i,j)+d(i-1,j))*isp2i
            if(j.gt.1)
     $       vtf(i,j)=va(i,j)*(d(i,j)+d(i,j-1))*isp2i
          end do
        end do
C
C-----------------------------------------------------------------------
C
        do 8000 iext=1,isplit    ! Begin external (2-D) mode 
C
          do j=2,jm
            do i=2,im
              fluxua(i,j)=.25e0*(d(i,j)+d(i-1,j))
     $                     *(dy(i,j)+dy(i-1,j))*ua(i,j)
              fluxva(i,j)=.25e0*(d(i,j)+d(i,j-1))
     $                     *(dx(i,j)+dx(i,j-1))*va(i,j)
            end do
          end do
C
C     NOTE addition of surface freshwater flux, w(i,j,1)=vflux, compared
C     with pom98.f. See also modifications to subroutine vertvl.
C
          do j=2,jmm1
            do i=2,imm1
              elf(i,j)=elb(i,j)
     $                  +dte2*(-(fluxua(i+1,j)-fluxua(i,j)
     $                          +fluxva(i,j+1)-fluxva(i,j))/art(i,j)
     $                          -vfluxf(i,j))
            end do
          end do
C
          call bcond(1)

          if(mod(iext,ispadv).eq.0) call advave(tps)
C
          do j=2,jmm1
            do i=2,im
              uaf(i,j)=adx2d(i,j)+advua(i,j)
     $                  -aru(i,j)*.25e0
     $                    *(cor(i,j)*d(i,j)*(va(i,j+1)+va(i,j))
     $                     +cor(i-1,j)*d(i-1,j)*(va(i-1,j+1)+va(i-1,j)))
     $                  +.25e0*grav*(dy(i,j)+dy(i-1,j))
     $                    *(d(i,j)+d(i-1,j))
     $                    *((1.e0-2.e0*alpha)
     $                       *(el(i,j)-el(i-1,j))
     $                      +alpha*(elb(i,j)-elb(i-1,j)
     $                             +elf(i,j)-elf(i-1,j))
     $                      +e_atmos(i,j)-e_atmos(i-1,j))
     $                  +drx2d(i,j)+aru(i,j)*(wusurf(i,j)-wubot(i,j))
            end do
          end do
C
          do j=2,jmm1
            do i=2,im
              uaf(i,j)=((h(i,j)+elb(i,j)+h(i-1,j)+elb(i-1,j))
     $                    *aru(i,j)*uab(i,j)
     $                  -4.e0*dte*uaf(i,j))
     $                 /((h(i,j)+elf(i,j)+h(i-1,j)+elf(i-1,j))
     $                     *aru(i,j))
            end do
          end do
C
          do j=2,jm
            do i=2,imm1
              vaf(i,j)=ady2d(i,j)+advva(i,j)
     $                  +arv(i,j)*.25e0
     $                    *(cor(i,j)*d(i,j)*(ua(i+1,j)+ua(i,j))
     $                     +cor(i,j-1)*d(i,j-1)*(ua(i+1,j-1)+ua(i,j-1)))
     $                  +.25e0*grav*(dx(i,j)+dx(i,j-1))
     $                    *(d(i,j)+d(i,j-1))
     $                    *((1.e0-2.e0*alpha)*(el(i,j)-el(i,j-1))
     $                      +alpha*(elb(i,j)-elb(i,j-1)
     $                             +elf(i,j)-elf(i,j-1))
     $                      +e_atmos(i,j)-e_atmos(i,j-1))
     $                  +dry2d(i,j)+arv(i,j)*(wvsurf(i,j)-wvbot(i,j))
            end do
          end do
C
          do j=2,jm
            do i=2,imm1
              vaf(i,j)=((h(i,j)+elb(i,j)+h(i,j-1)+elb(i,j-1))
     $                    *vab(i,j)*arv(i,j)
     $                  -4.e0*dte*vaf(i,j))
     $                 /((h(i,j)+elf(i,j)+h(i,j-1)+elf(i,j-1))
     $                     *arv(i,j))
            end do
          end do
C
          call bcond(2)
C
          if(iext.eq.(isplit-2))then
            do j=1,jm
              do i=1,im
                etf(i,j)=.25e0*smoth*elf(i,j)
              end do
            end do
C
          else if(iext.eq.(isplit-1)) then
C
            do j=1,jm
              do i=1,im
                etf(i,j)=etf(i,j)+.5e0*(1.-.5e0*smoth)*elf(i,j)
              end do
            end do
C
          else if(iext.eq.isplit) then
C
            do j=1,jm
              do i=1,im
                etf(i,j)=(etf(i,j)+.5e0*elf(i,j))*fsm(i,j)
              end do
            end do
C


      endif

C  TEST FOR CFL VIOLATION. IF SO, PRINT AND STOP

      VAMAX=0.       
      DO 442 J=1,JM
      DO 442 I=1,IM
C      IF(ABS(VAF(I,J)).GE.VAMAX)VAMAX=ABS(VAF(I,J))   
      IF(SQRT(VAF(I,J)**2+UAF(I,J)**2).GE.VAMAX) 
     $     VAMAX=SQRT(VAF(I,J)**2+UAF(I,J)**2)   
  442 CONTINUE
      IF(VAMAX.GT.100.) GO TO 9001

C*** CALCULATE TERMS IN VORTICITY BALANCE AND WRITE TO UNIT 95 - DJS 02/01

C      IF(IEXT.EQ.ISPLIT.AND.MOD(IFIX(IINT*DTI),3600).EQ.0) THEN
C       CALL VORT(ADVUA,ADVVA,ADVUU,ADVVV,TRNU,TRNV,IM,JM,0)
C       CALL VORT(ADVUA,ADVVA,ADVUU,ADVVV,TRNU,TRNV,IM,JM,1)
C      ENDIF


C
C     Apply filter to remove time split and reset time sequence:
C
            do j=1,jm
              do i=1,im
                ua(i,j)=ua(i,j)
     $                   +.5e0*smoth*(uab(i,j)-2.e0*ua(i,j)+uaf(i,j))
                va(i,j)=va(i,j)
     $                   +.5e0*smoth*(vab(i,j)-2.e0*va(i,j)+vaf(i,j))
                el(i,j)=el(i,j)
     $                   +.5e0*smoth*(elb(i,j)-2.e0*el(i,j)+elf(i,j))
                elb(i,j)=el(i,j)
                el(i,j)=elf(i,j)
                d(i,j)=h(i,j)+el(i,j)
                uab(i,j)=ua(i,j)
                ua(i,j)=uaf(i,j)
                vab(i,j)=va(i,j)
                va(i,j)=vaf(i,j)
              end do
            end do
C
            if(iext.ne.isplit) then
              do j=1,jm
                do i=1,im
                  egf(i,j)=egf(i,j)+el(i,j)*ispi
                  if(i.gt.1)
     $             utf(i,j)=utf(i,j)+ua(i,j)*(d(i,j)+d(i-1,j))*isp2i
                  if(j.gt.1)
     $             vtf(i,j)=vtf(i,j)+va(i,j)*(d(i,j)+d(i,j-1))*isp2i
                end do
              end do
            endif
C

 8000                    CONTINUE
C---------------------------------------------------------------------
C          END EXTERNAL (2-D) MODE CALCULATION
C     AND CONTINUE WITH INTERNAL (3-D) MODE CALCULATION
C---------------------------------------------------------------------

      IF(MODE.EQ.2) GO TO 8200

C  IF THIS IS NOT A TRUE RESTART, SKIP FIRST INTERNAL TIME STEP

      IF(nread.EQ.0.AND.IINT.EQ.1) GO TO 8200

C---------------------------------------------------------------------
C      ADJUST U(Z) AND V(Z) SUCH THAT
C      VERTICAL AVERAGE OF (U,V) = (UA,VA)
C---------------------------------------------------------------------
C
C
C     Adjust u(z) and v(z) such that depth average of (u,v) = (ua,va):
C
            do j=1,jm
              do i=1,im
                tps(i,j)=0.e0
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  tps(i,j)=tps(i,j)+u(i,j,k)*dz(k)
                end do
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=2,im
                  u(i,j,k)=(u(i,j,k)-tps(i,j))+
     $                     (utb(i,j)+utf(i,j))/(dt(i,j)+dt(i-1,j))
                end do
              end do
            end do
C
            do j=1,jm
              do i=1,im
                tps(i,j)=0.e0
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  tps(i,j)=tps(i,j)+v(i,j,k)*dz(k)
                end do
              end do
            end do
C
            do k=1,kbm1
              do j=2,jm
                do i=1,im
                  v(i,j,k)=(v(i,j,k)-tps(i,j))+
     $                     (vtb(i,j)+vtf(i,j))/(dt(i,j)+dt(i,j-1))
                end do
              end do
            end do
C
C     vertvl calculates w from u, v, dt (h+et), etf and etb:
C
            call vertvl(a,c)
            call bcond(5)
C
C
            do k=1,kb
              do j=1,jm
                do i=1,im
                  uf(i,j,k)=0.e0
                  vf(i,j,k)=0.e0
                end do
              end do
            end do

      IF(YAMADA.EQ.0.) THEN     
C
C     Calculate q2f and q2lf using uf, vf, a and c as temporary
C     variables:
C
            call advq(q2b,q2,uf,a,c)
            call advq(q2lb,q2l,vf,a,c)
            call profq(a,c,tps,dtef)
            call bcond(6)
C
            do k=1,kb
              do j=1,jm
                do i=1,im
                  q2(i,j,k)=q2(i,j,k)
     $                       +.5e0*smoth*(uf(i,j,k)+q2b(i,j,k)
     $                                    -2.e0*q2(i,j,k))
                  q2l(i,j,k)=q2l(i,j,k)
     $                       +.5e0*smoth*(vf(i,j,k)+q2lb(i,j,k)
     $                                    -2.e0*q2l(i,j,k))
                  q2b(i,j,k)=q2(i,j,k)
                  q2(i,j,k)=uf(i,j,k)
                  q2lb(i,j,k)=q2l(i,j,k)
                  q2l(i,j,k)=vf(i,j,k)
                end do
              end do
            end do
  
      ENDIF

      IF(YAMADA.LT.0.) THEN     

      DO 313 K=1,KB
      DO 313 J=1,JMM1
      DO 313 I=1,IMM1
      IF(FSM(I,J).EQ.0.E0) GO TO 313
      KM(I,J,K)=0.0025-YAMADA*DTE*SQRT(
     1   ((WUSURF(I,J)+WUSURF(I+1,J))/(DUM(I,J)+DUM(I+1,J)+1.E-10))**2 +
     2   ((WVSURF(I,J)+WVSURF(I,J+1))/(DVM(I,J)+DVM(I,J+1)+1.E-10))**2)
      KH(I,J,K)=KM(I,J,K)/10.
  313 CONTINUE
      
      ENDIF

C
C     Calculate tf and sf using uf, vf, a and c as temporary variables:
C
            if(mode.ne.4) then
C
              if(nadv.eq.1) then
C
                call advt1(tb,t,tmean,uf,a,c)
C
              else if(nadv.eq.2) then
C
                call advt2(tb,t,tmean,uf,a,c,nitera,sw)
C
              else
C
                write(6,9)
    9           format(/'Invalid value for nadv ..... ',
     $                 'program terminated'/)
                stop
C
              endif


C  PENETRATIVE SW RADIATION

      IF(nbct.EQ.2) THEN
       DO 311 J=1,JM
       DO 311 I=1,IM
        WTSURF(I,J)=WTSURF(I,J)-SWRAD(I,J)
  311  CONTINUE
      ENDIF

              call proft(uf,wtsurf,tsurf,nbct,tps)

C  RESTORE WTSURF FOR OUTPUT PURPOSES
 
      IF(nbct.EQ.2) THEN
       DO 312 J=1,JM
       DO 312 I=1,IM
        WTSURF(I,J)=WTSURF(I,J)+SWRAD(I,J)
  312  CONTINUE
      ENDIF

C              call proft(vf,wssurf,ssurf,nbcs,tps)
              call bcond(4)
C
              do k=1,kb
                do j=1,jm
                  do i=1,im
                    t(i,j,k)=t(i,j,k)
     $                        +.5e0*smoth*(uf(i,j,k)+tb(i,j,k)
     $                                     -2.e0*t(i,j,k))
                    s(i,j,k)=s(i,j,k)
     $                        +.5e0*smoth*(vf(i,j,k)+sb(i,j,k)
     $                                     -2.e0*s(i,j,k))
                    tb(i,j,k)=t(i,j,k)
                    t(i,j,k)=uf(i,j,k)
                    sb(i,j,k)=s(i,j,k)
                    s(i,j,k)=vf(i,j,k)


C  LIMIT TEMPERATURE TO A MINIMUM OF 0 C

      T(I,J,K)=AMAX1(T(I,J,K),-TBIAS)
      TB(I,J,K)=AMAX1(TB(I,J,K),-TBIAS)

                  end do
                end do
              end do

      
C
      CALL DENS(S,T,RHO)
C
            endif
C
C

C
C     Calculate uf and vf:
C
            call advu
            call advv
            call profu
            call profv
            call bcond(3)
C
            do j=1,jm
              do i=1,im
                tps(i,j)=0.e0
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  tps(i,j)=tps(i,j)
     $                      +(uf(i,j,k)+ub(i,j,k)-2.e0*u(i,j,k))*dz(k)
                end do
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  u(i,j,k)=u(i,j,k)
     $                      +.5e0*smoth*(uf(i,j,k)+ub(i,j,k)
     $                                   -2.e0*u(i,j,k)-tps(i,j))
                end do
              end do
            end do
C
            do j=1,jm
              do i=1,im
                tps(i,j)=0.e0
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  tps(i,j)=tps(i,j)
     $                      +(vf(i,j,k)+vb(i,j,k)-2.e0*v(i,j,k))*dz(k)
                end do
              end do
            end do
C
            do k=1,kbm1
              do j=1,jm
                do i=1,im
                  v(i,j,k)=v(i,j,k)
     $                      +.5e0*smoth*(vf(i,j,k)+vb(i,j,k)
     $                                   -2.e0*v(i,j,k)-tps(i,j))
                end do
              end do
            end do
C
            do k=1,kb
              do j=1,jm
                do i=1,im
                  ub(i,j,k)=u(i,j,k)
                  u(i,j,k)=uf(i,j,k)
                  vb(i,j,k)=v(i,j,k)
                  v(i,j,k)=vf(i,j,k)
                end do
              end do
            end do
C
C          endif


C
 8200 CONTINUE
C
          do j=1,jm
            do i=1,im
              egb(i,j)=egf(i,j)
              etb(i,j)=et(i,j)
              et(i,j)=etf(i,j)
              dt(i,j)=h(i,j)+et(i,j)
              utb(i,j)=utf(i,j)
              vtb(i,j)=vtf(i,j)
              vfluxb(i,j)=vfluxf(i,j)
            end do
          end do

C-----------------------------------------------------------------------
C RECALCULATE TMEAN IF REQUESTED
C-----------------------------------------------------------------------

      IF(ITMEAN.GT.0.AND.MOD(IINT,ITMEAN).EQ.0) CALL TZAVG

 9001 CONTINUE

C-----------------------------------------------------------------------
C WRITE A ONE TIME LEVEL RESTART FILE 
C     WRITE(80) TIME,UB,VB,UAB,VAB,TB,SB,ELB,Q2B,Q2LB,KM,KH
C---------------------------------------------------------------------------

      IF(VAMAX.GT.100.) THEN
        CALL PRXY('  EL ',TIME, EL ,IP,ISK,JM,1,0.E0)
        CALL PRXY('  ELF',TIME, ELF,IP,ISK,JM,1,0.E0)
        CALL PRXY('  UA            ',TIME,UA ,IP,ISK,JM,1,0.E0)
        CALL PRXY('  VA            ',TIME,VA ,IP,ISK,JM,1,0.E0)
        CALL PRXY('  UAF           ',TIME,UAF,IP,ISK,JM,1,0.E0)
        CALL PRXY('  VAF           ',TIME,VAF,IP,ISK,JM,1,0.E0)
       WRITE(6,'(//////////////////)')
       WRITE(6,'(''************************************************'')')
       WRITE(6,'(''************ ABNORMAL JOB END ******************'')')
       WRITE(6,'(''************* USER TERMINATED ******************'')')
       WRITE(6,'(''************************************************'')')
       WRITE(6,'('' VAMAX ='',E12.3)') VAMAX
       WRITE(6,'('' IINT,IEXT ='',2I6)') IINT,IEXT
       STOP
      ENDIF
 7000 CONTINUE

C UPDATE TIME

      ISEC=ISEC+IFIX(DTI)
      IF(ISEC.GE.3600) THEN
       I=MOD(ISEC,3600)
       IHR0=IHR0+(ISEC-I)/3600
       ISEC=I
      ENDIF
      IF (IHR0.GE.24) THEN
       IHR0=IHR0-24
       IJD0=IJD0+1
       IF(IJD0.GT.NDYR) THEN
        IJD0=IJD0-NDYR
        IYR0=IYR0+1
        IF(MOD(IYR0,4).EQ.0) THEN
         NDYR=366
        ELSE
         NDYR=365
        ENDIF
       ENDIF
      ENDIF

C-----------------------------------------------------------------------
C CALL OUTPUT SUBROUTINE AT EVERY INTERNAL MODE TIME STEP  TO PRINT 
C VARIOUS OUTPUT AT SPECIFIED INTERVALS. 
C-----------------------------------------------------------------------

      CALL OUTP3D

C WRITE A TWO TIME LEVEL (SEAMLESS) MULTIPLE RESTART FILE 

      IF(NRSTRT.GT.0.AND.MOD(IINT,NRSTRT).EQ.0) CALL RSTART(2)
     

 9000                     CONTINUE
C***********************************************************************
C                                                                      *
C                  END NUMERICAL INTEGRATION                           *
C                                                                      *
C***********************************************************************

C WRITE A TWO TIME LEVEL (SEAMLESS) RESTART FILE 

      CALL RSTART(1)

      STOP
      END
